package com.zettenet.recipesdb.model;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Table;

import org.hibernate.annotations.Proxy;

import com.zettenet.recipesdb.web.models.INamedComparablePersistentObject;

/**
 * @author pvzetten
 */
@Entity
@Table(name = "recipe_category")
@Proxy(lazy = false)
public class RecipeCategory implements Serializable, INamedComparablePersistentObject<RecipeCategory> {
	private static final long serialVersionUID = -2667690102180052285L;

	/**
	 * The name of the category
	 */
	@Column(unique = true)
	private String name;

	/**
	 * The unique identifier of the category.
	 */
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	private Long id;

	/**
	 * Constructor.
	 */
	public RecipeCategory() {
	}

	/**
	 * Constructor.
	 * 
	 * @param category
	 *            the name of the category
	 */
	public RecipeCategory(String name) {
		this.name = name;
	}

	@Override
	public int compareTo(RecipeCategory that) {
		if (that == null) {
			return -1;
		}

		return name.compareTo(that.name);
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		
		if (obj == null) {
			return false;
		}
		
		if (getClass() != obj.getClass()) {
			return false;
		}
		
		RecipeCategory other = (RecipeCategory) obj;
		
		if (name == null) {
			if (other.name != null) {
				return false;
			}
		} else if (!name.equals(other.name)) {
			return false;
		}
		
		return true;
	}

	/**
	 * @return the name of the category
	 */
	public String getName() {
		return name;
	}

	/**
	 * Gets the unique identifier of this category.
	 * 
	 * @return long version of the serializable id stored in the database
	 */
	public Long getId() {
		return id;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		return result;
	}

	/**
	 * @param category
	 *            the name of the category
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Sets the unique identifier of this object. Should not be used manually.
	 */
	public void setId(Long id) {
		this.id = id;
	}

	@Override
	public String toString() {
		String ret = "[";

		ret += id + ", ";
		ret += "\"" + name + "\"]";

		return ret;
	}
}
