package com.zettenet.recipesdb.model;

import java.io.Serializable;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.UniqueConstraint;

import org.hibernate.annotations.Proxy;

/**
 * A link between a Recipe and a RecipeCategory.
 * 
 * @author pvzetten
 */
@Entity
@Table(name = "recipe_category_link", uniqueConstraints = { @UniqueConstraint(columnNames = {
		"recipe_id", "category_id" }) })
@Proxy(lazy = false)
public class RecipeCategoryEntry implements Serializable, Comparable<RecipeCategoryEntry> {
	private static final long serialVersionUID = -3467001483616322743L;

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	private Long id;

	@ManyToOne
	@PrimaryKeyJoinColumn(name = "recipe_id")
	private Recipe recipe;

	@ManyToOne
	@PrimaryKeyJoinColumn(name = "category_id")
	private RecipeCategory category;

	public RecipeCategoryEntry() {
	}

	public RecipeCategoryEntry(Recipe recipe, RecipeCategory category) {
		this.recipe = recipe;
		this.category = category;
	}

	public Long getId() {
		return this.id;
	}

	@SuppressWarnings("unused")
	private void setId(Long id) {
		this.id = id;
	}

	public Recipe getRecipe() {
		return this.recipe;
	}

	public void setRecipe(Recipe recipe) {
		this.recipe = recipe;
	}

	public RecipeCategory getCategory() {
		return this.category;
	}

	public void setCategory(RecipeCategory category) {
		this.category = category;
	}

	@Override
	public String toString() {
		return "Tag: " + this.category.toString();
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}

		if (obj == null) {
			return false;
		}

		if (getClass() != obj.getClass()) {
			return false;
		}

		RecipeCategoryEntry other = (RecipeCategoryEntry) obj;

		if (this.recipe == null) {
			if (this.recipe.getId() != other.recipe.getId()) {
				return false;
			}
		}

		if (this.category == null) {
			if (!this.category.equals(other.getCategory())) {
				return false;
			}
		}

		return true;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((this.id == null) ? 0 : this.id.hashCode());
		result = (int) (prime * result + ((this.recipe == null) ? 0
				: (this.recipe.getId() == null ? 0 : this.recipe.getId())));
		result = prime * result + ((category == null) ? 0 : category.hashCode());
		return result;
	}

	@Override
	public int compareTo(RecipeCategoryEntry that) {
		if (that == null) {
			return -1;
		}

		if (this == that) {
			return 0;
		}

		return this.category.compareTo(that.category);
	}

}
