package com.zettenet.recipesdb.model;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Table;

import com.zettenet.recipesdb.web.models.INamedComparablePersistentObject;

/**
 * An ingredient to be used in recipes.
 * 
 * @author pvzetten
 */
@Entity
@Table(name = "recipe_ingredient")
public class RecipeIngredient implements Serializable, INamedComparablePersistentObject<RecipeIngredient> {
	private static final long serialVersionUID = -8352989852300447295L;

	/**
	 * The unique identifier of the ingredient.
	 */
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	private Long id;

	/**
	 * The name of the ingredient.
	 */
	@Column(unique = true)
	private String name;

	/**
	 * Constructor.
	 */
	public RecipeIngredient() {
	}

	/**
	 * Constructor.
	 * 
	 * @param ingredient
	 *            the name of the ingredient
	 */
	public RecipeIngredient(String name) {
		this.name = name;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		RecipeIngredient other = (RecipeIngredient) obj;
		if (name == null) {
			if (other.name != null) {
				return false;
			}
		} else if (!name.equals(other.name)) {
			return false;
		}
		return true;
	}

	/**
	 * Gets the unique identifier of the ingredient.
	 * 
	 * @return long version of the serializable id stored in the database.
	 */
	public Long getId() {
		return id;
	}

	/**
	 * @return the name of the ingredient
	 */
	public String getName() {
		return name;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		return result;
	}

	/**
	 * Sets the unique identifier of this object. Should not be used manually.
	 */
	@SuppressWarnings("unused")
	private void setId(Long id) {
		this.id = id;
	}

	/**
	 * @param ingredient
	 *            the name of the ingredient
	 */
	public void setName(String ingredient) {
		this.name = ingredient;
	}

	@Override
	public String toString() {
		String ret = "[";

		ret += id + ", ";
		ret += "\"" + name + "\"]";

		return ret;
	}

	@Override
	public int compareTo(RecipeIngredient that) {
		if (that == null) {
			return -1;
		}

		return name.compareTo(that.name);
	}

}
