package com.zettenet.recipesdb.model;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Table;

import com.zettenet.recipesdb.web.models.INamedComparablePersistentObject;

/**
 * A category to contain lists of ingredients; e.g., "For sauce", or "To serve".
 * 
 * @author pvzetten
 */
@Entity
@Table(name = "recipe_ingredient_group")
public class RecipeIngredientGroup implements Serializable, INamedComparablePersistentObject<RecipeIngredientGroup> {
	private static final long serialVersionUID = 2710280269154761591L;

	public static final String DEFAULT_GROUP = "Main";

	/**
	 * The unique identifier of the ingredient group.
	 */
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	private Long id;

	/**
	 * The name of the group.
	 */
	@Column(unique = true)
	private String name;

	/**
	 * Constructor.
	 */
	public RecipeIngredientGroup() {
	}

	/**
	 * Constructor.
	 * 
	 * @param group
	 *            the name of the group
	 */
	public RecipeIngredientGroup(String name) {
		this.name = name;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		
		if (obj == null) {
			return false;
		}
		
		if (getClass() != obj.getClass()) {
			return false;
		}
		
		RecipeIngredientGroup other = (RecipeIngredientGroup) obj;
		if (name == null) {
			if (other.name != null) {
				return false;
			}
		} else if (!name.equals(other.name)) {
			return false;
		}
		
		return true;
	}

	/**
	 * Gets the unique identifier of this source.
	 * 
	 * @return long version of the serializable id stored in the database
	 */
	public Long getId() {
		return id;
	}

	/**
	 * @return the name of the group
	 */
	public String getName() {
		return name;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		return result;
	}

	/**
	 * Sets the unique identifier of this object. Should not be used manually.
	 */
	@SuppressWarnings("unused")
	private void setId(Long id) {
		this.id = id;
	}

	/**
	 * @param group
	 *            the name of the group
	 */
	public void setName(String name) {
		this.name = name;
	}

	@Override
	public String toString() {
		String ret = "[";

		ret += id + ", ";
		ret += "\"" + name + "\"]";

		return ret;
	}

	@Override
	public int compareTo(RecipeIngredientGroup that) {
		if (that == null) {
			return -1;
		}

		return name.compareTo(that.name);
	}
}
