package com.zettenet.recipesdb.persistence;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

import com.zettenet.recipesdb.model.Recipe;
import com.zettenet.recipesdb.model.RecipeCategory;
import com.zettenet.recipesdb.model.RecipeIngredient;
import com.zettenet.recipesdb.model.RecipeIngredientEntry;
import com.zettenet.recipesdb.model.RecipeIngredientGroup;
import com.zettenet.recipesdb.model.RecipeMethod;
import com.zettenet.recipesdb.model.RecipeSource;
import com.zettenet.recipesdb.model.RecipeStep;

public class OldDBImport {
	private static final String url = "jdbc:mysql://localhost/recipes";
	Connection conn;
	ResultSet resultSet;
	Statement statement;
	
	public OldDBImport() {
		try {
			Class.forName("com.mysql.jdbc.Driver");
			conn = DriverManager.getConnection(url, "recipesdb_user", "recipesdb_user");
			statement = conn.createStatement();
			resultSet = statement.executeQuery(
					"SELECT r.id, r.name, r.servings, r.source, r.method " +
					"FROM recipe r " +
					"ORDER BY r.id ASC");
			
			processRecipes();
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		close();
	}
	
	private void processRecipes() {
		
		try {
			while (resultSet.next()) {
				String id = resultSet.getString("id");
				
				if (id != "43") {
					String name = resultSet.getString("name");
					String servings = resultSet.getString("servings");
					String source = resultSet.getString("source");
					String category = "Imported";
					
					String sMethod = resultSet.getString("method");
					
					String regex = "(</p>)*\\s*(<p>)";
					
					String[] methodSteps = sMethod.split(regex);
					
					Recipe recipe = new Recipe();
					recipe.setName(name);
					recipe.setServings(servings);
					recipe.setSource(new RecipeSource(source));
					recipe.addCategory(new RecipeCategory(category));
	
					RecipeMethod method = new RecipeMethod();
					method.setNumberedSteps(true);
					
					List<RecipeStep> steps = new ArrayList<RecipeStep>();
					
					int i = 0;

					for (String s : methodSteps) {
						if (s != null && !s.trim().equals("")) {
							s = s.trim().replace("</p>", "");
							steps.add(new RecipeStep(i, s));
							i++;
						}
					}
					
					method.setSteps(steps);
					recipe.setMethod(method);
					
					List<RecipeIngredientEntry> ingredients = new ArrayList<RecipeIngredientEntry>();
					
					int j = 0;
					
					Statement inner = conn.createStatement();
					ResultSet innerResult = inner.executeQuery(
							"SELECT l.amount, i.ingredient, c.category " +
							"FROM ingrlink l, ingredients i, categories c " +
							"WHERE l.recipeid='" + id + "' "+
							"	AND i.ingredient_id=l.ingid "+
							"	AND c.category_id=l.catid "+
							"ORDER BY l.seq ASC");
					
					while (innerResult.next()) {
						String amount = innerResult.getString("amount");
						RecipeIngredient ingredient = new RecipeIngredient(innerResult.getString("ingredient"));
						RecipeIngredientGroup group = new RecipeIngredientGroup(innerResult.getString("category"));
						
						ingredients.add(new RecipeIngredientEntry(j, amount, ingredient, group));
						j++;
					}
					
					recipe.setIngredients(ingredients);
					
					//this.dao.saveOrUpdate(recipe);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private void close() {
		try {
			if (resultSet != null) {
				resultSet.close();
			}

			if (statement != null) {
				statement.close();
			}

			if (conn != null) {
				conn.close();
			}
		} catch (Exception e) {

		}
	}

}
