package com.zettenet.recipesdb.persistence.hibernate;

import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Restrictions;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;

import com.zettenet.recipesdb.persistence.IPersistentUniqueNamedObjectDao;
import com.zettenet.recipesdb.services.impl.InvalidDataException;
import com.zettenet.recipesdb.web.models.INamedComparablePersistentObject;

public abstract class AbstractRecipeComponentDaoHibernate<T extends INamedComparablePersistentObject<T>> extends HibernateDaoSupport implements IPersistentUniqueNamedObjectDao<T> {
	
	protected Class<T> clazz;
	
	public AbstractRecipeComponentDaoHibernate(Class<T> clazz) {
		this.clazz = clazz;
	}

	@SuppressWarnings("unchecked")
	protected List<T> findByCriteria(Criterion... criteria) {
		Criteria crit = getSession().createCriteria(this.clazz);
		crit.setResultTransformer(Criteria.DISTINCT_ROOT_ENTITY);
		
		if (criteria != null) {
			for (Criterion c : criteria) {
				crit.add(c);
			}
		}
		
		return crit.list();
	}
	
	@SuppressWarnings("unchecked")
	protected T getById(Long id) {
		return (T) getSession().get(this.clazz, id);
	}

	@Override
	public List<T> findAll() {
		return findByCriteria((Criterion[]) null);
	}

	@Override
	public List<T> find(String name, boolean inclusive) {
		Criterion crit = Restrictions.ilike("name", name,
				inclusive ? MatchMode.ANYWHERE : MatchMode.EXACT);
		return findByCriteria(crit);
	}

	@Override
	public T get(Long id) {
		return getById(id);
	}

	@Override
	public T getOrSave(String name) throws InstantiationException, IllegalAccessException, InvalidDataException {
		T result;
		List<T> match = find(name, false);
		
		if (match.size() == 0) {
			result = this.clazz.newInstance();
			result.setName(name);
			getSession().save(result);
		} else if (match.size() == 1) {
			result = match.get(0);
		} else {
			// TODO: Log a non-fatal error - we have more than one object with the same name
			result = match.get(0);
		}
		
		return result;
	}
	
}
