package com.zettenet.recipesdb.services;

import java.util.List;
import java.util.Set;

import com.zettenet.recipesdb.model.Recipe;
import com.zettenet.recipesdb.model.RecipeCategory;
import com.zettenet.recipesdb.model.RecipeIngredient;
import com.zettenet.recipesdb.model.RecipeIngredientGroup;
import com.zettenet.recipesdb.model.RecipeSource;
import com.zettenet.recipesdb.services.impl.InvalidClientException;
import com.zettenet.recipesdb.services.impl.InvalidDataException;


public interface IRecipesDataService {
	// Searching
	List<Recipe> getAllRecipes(String clientToken) throws InvalidClientException;
	List<Recipe> getRecipesByName(String clientToken, String name) throws InvalidClientException;
	List<Recipe> getRecipesByName(String clientToken, String name, boolean inclusive) throws InvalidClientException;
	List<Recipe> getRecipesBySource(String clientToken, RecipeSource source) throws InvalidClientException, InvalidDataException, InstantiationException, IllegalAccessException;
	List<Recipe> getRecipesByTag(String clientToken, Set<RecipeCategory> tags) throws InvalidClientException;
	List<Recipe> getRecipesByTag(String clientToken, Set<RecipeCategory> tags, boolean inclusive) throws InvalidClientException;
	List<Recipe> getRecipesByIngredient(String clientToken, Set<RecipeIngredient> ingredients) throws InvalidClientException;
	List<Recipe> getRecipesByIngredient(String clientToken, Set<RecipeIngredient> ingredients, boolean inclusive) throws InvalidClientException;
	
	List<RecipeSource> getAllSources(String clientToken) throws InvalidClientException;
	List<RecipeSource> getSourcesByName(String clientToken, String name) throws InvalidClientException;
	List<RecipeSource> getSourcesByName(String clientToken, String name, boolean inclusive) throws InvalidClientException;
	
	List<RecipeIngredient> getAllIngredients(String clientToken) throws InvalidClientException;
	List<RecipeIngredient> getIngredientsByName(String clientToken, String name) throws InvalidClientException;
	List<RecipeIngredient> getIngredientsByName(String clientToken, String name, boolean inclusive) throws InvalidClientException;
	
	List<RecipeCategory> getAllTags(String clientToken) throws InvalidClientException;
	List<RecipeCategory> getTagsByName(String clientToken, String name) throws InvalidClientException;
	List<RecipeCategory> getTagsByName(String clientToken, String name, boolean inclusive) throws InvalidClientException;
	
	// Retrieving individual entities
	Recipe getRecipe(String clientToken, Long id) throws InvalidClientException, InvalidDataException;
	RecipeSource getSource(String clientToken, Long id) throws InvalidClientException, InvalidDataException;
	RecipeSource getOrSaveSource(String clientToken, String name) throws InvalidClientException, InvalidDataException, InstantiationException, IllegalAccessException;
	RecipeCategory getTag(String clientToken, Long id) throws InvalidClientException, InvalidDataException;
	RecipeCategory getOrSaveTag(String clientToken, String name) throws InvalidClientException, InvalidDataException, InstantiationException, IllegalAccessException;
	RecipeIngredient getIngredient(String clientToken, Long id) throws InvalidClientException, InvalidDataException;
	RecipeIngredient getOrSaveIngredient(String clientToken, String name) throws InvalidClientException, InvalidDataException, InstantiationException, IllegalAccessException;
	RecipeIngredientGroup getIngredientGroup(String clientToken, Long id) throws InvalidClientException, InvalidDataException;
	RecipeIngredientGroup getOrSaveIngredientGroup(String clientToken, String name) throws InvalidClientException, InvalidDataException, InstantiationException, IllegalAccessException;
	
	// Saving entities
	Long saveOrUpdateRecipe(String clientToken, Recipe recipe) throws InvalidClientException, InvalidDataException, InstantiationException, IllegalAccessException;
	
	// Miscellaneous updating
	void tagRecipes(String clientToken, RecipeCategory tag, Set<Recipe> recipes) throws InvalidClientException, InvalidDataException, InstantiationException, IllegalAccessException;
	void untagRecipes(String clientToken, RecipeCategory tag, Set<Recipe> recipes) throws InvalidClientException, InvalidDataException, InstantiationException, IllegalAccessException;
	void cleanupDatabase(String clientToken) throws InvalidClientException, InvalidDataException;
}
