package com.zettenet.recipesdb.web.components;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.wicket.Component;
import org.apache.wicket.Page;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;

import com.zettenet.recipesdb.web.models.INamedComparablePersistentObject;

public class AlphabeticalListView<T extends INamedComparablePersistentObject<T>> extends
		Panel {
	private static final long serialVersionUID = -3703411061199158353L;
	
	private Map<Character, Component> usedInitials = new HashMap<Character, Component>();
	private IModel<List<T>> model;
	private Class<? extends Page> itemPage;
	private String listId;

	public AlphabeticalListView(String id, IModel<List<T>> model,
			Class<? extends Page> itemPage, String listId) {
		super(id, model);

		this.model = model;
		this.itemPage = itemPage;
		this.listId = listId;

		addItemList();
		addAlphabetTable();
	}

	private void addAlphabetTable() {
		List<Character> alphabet = new ArrayList<Character>();
		for (char c = 'A'; c <= 'Z'; ++c) {
			alphabet.add(c);
		}

		ListView<Character> alphalinks = new ListView<Character>("alphalinks",
				alphabet) {
			private static final long serialVersionUID = -5136774580654457172L;
			Map<Character, Component> usedInitials = getUsedInitials();

			@Override
			protected void populateItem(ListItem<Character> item) {

				Character letter = item.getModelObject();
				BookmarkablePageLink<?> link = new BookmarkablePageLink<Object>("alphalink", this.getPage().getClass());

				if (this.usedInitials.containsKey(letter)) {
					link.setAnchor(this.usedInitials.get(letter));
				}

				Label alphalabel = new Label("alphalabel", letter.toString());
				alphalabel.setRenderBodyOnly(true);
				link.add(alphalabel);

				item.add(link);
			}
		};

		add(alphalinks);
	}

	private void addItemList() {
		WebMarkupContainer itemsList = new WebMarkupContainer("itemsList");

		if ((this.listId != null) && !this.listId.equals("")) {
			itemsList.setOutputMarkupId(true);
			itemsList.setMarkupId(this.listId);
		} else {
			itemsList.setOutputMarkupId(false);
		}

		ListView<T> items = new ListView<T>("allItems", this.model) {
			private static final long serialVersionUID = 225522409658868078L;
			private Character oldInitial = ' ';

			@Override
			protected void populateItem(ListItem<T> item) {
				T object = item.getModelObject();
				String name = object.getName();
				Character initial = name.charAt(0);
				Long id = object.getId();

				Label itemAlpha = new Label("itemAlpha", initial.toString());
				itemAlpha.setOutputMarkupId(true);
				itemAlpha.setMarkupId("alpha" + initial.toString());

				if (initial.equals(this.oldInitial)) {
					itemAlpha.setVisible(false);
				} else {
					addUsedInitial(initial, itemAlpha);
					this.oldInitial = initial;
				}

				item.add(itemAlpha);

				BookmarkablePageLink<Page> itemLink = new BookmarkablePageLink<Page>(
						"itemLink", AlphabeticalListView.this.itemPage);
				itemLink.setParameter("view", id);

				Label itemName = new Label("itemName", name);
				itemName.setRenderBodyOnly(true);
				itemLink.add(itemName);

				item.add(itemLink);
			}
		};

		itemsList.add(items);
		add(itemsList);
	}

	protected void addUsedInitial(Character initial, Component component) {
		this.usedInitials.put(initial, component);
	}

	protected Map<Character, Component> getUsedInitials() {
		return this.usedInitials;
	}
}
