package com.zettenet.recipesdb.web.components.formFields;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.model.IModel;

import com.zettenet.recipesdb.web.models.INamedComparablePersistentObject;

public class CommaSeparatedSetTextField<T extends INamedComparablePersistentObject<T>>
		extends TextField<Collection<T>> {
	private static final long serialVersionUID = 7462669024599345604L;
	
	private final Class<T> itemClass;

	public CommaSeparatedSetTextField(String id, Class<T> itemClass) {
		super(id);
		this.itemClass = itemClass;
	}

	public CommaSeparatedSetTextField(String id, IModel<Collection<T>> model,
			Class<T> itemClass) {
		super(id, model);
		this.itemClass = itemClass;
	}

	@Override
	protected void convertInput() {
		Set<T> items = new HashSet<T>();

		String[] inputs = getValue().split(", ");

		for (String input : inputs) {
			try {
				T item = this.itemClass.newInstance();
				item.setName(input);
				items.add(item);
			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			}
		}

		setConvertedInput(items);
	}

	@Override
	protected String getModelValue() {
		String value = "";

		if (getModel().getObject() != null) {
			List<T> items = new ArrayList<T>(getModel().getObject());
			Collections.sort(items);
	
			for (T item : items) {
				String name = item.getName();
				value += name + ", ";
			}
			
			if (value != "") {
				value = value.substring(0, value.length() - 2);
			}
		}
		
		return value;
	}
}
