package com.zettenet.recipesdb.web.components.formFields;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.model.IModel;

import com.zettenet.recipesdb.web.models.IValuedOrderedComparablePersistentObject;

public class NewlineSeparatedListTextArea<T extends IValuedOrderedComparablePersistentObject<T>>
		extends TextArea<Collection<T>> {
	private static final long serialVersionUID = -42949877129408047L;
	
	private final Class<T> itemClass;

	public NewlineSeparatedListTextArea(String id, Class<T> itemClass) {
		super(id);
		this.itemClass = itemClass;
	}

	public NewlineSeparatedListTextArea(String id, IModel<Collection<T>> model,
			Class<T> itemClass) {
		super(id, model);
		this.itemClass = itemClass;
	}

	@Override
	protected void convertInput() {
		List<T> items = new ArrayList<T>();

		String[] inputs = getValue().split("[\\r?\\n|\\r]+");
		int sequence = 0;

		for (String input : inputs) {
			try {
				T item = this.itemClass.newInstance();
				item.setValue(input.trim(), sequence);
				items.add(item);
				sequence++;
			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			}
		}

		setConvertedInput(items);
	}

	@Override
	protected String getModelValue() {
		String value = "";
		
		if (getModel().getObject() != null) {
			List<T> items = new ArrayList<T>(getModel().getObject());
			String eol = System.getProperty("line.separator");
		
			for (T item : items) {
				String name = item.getValue();
				value += name + eol + eol;
			}
		
			value = value.trim();
		}
		
		return value;
	}

}
