package com.zettenet.recipesdb.web.components.pageContents;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Set;

import org.apache.wicket.markup.html.form.Button;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;

import com.zettenet.recipesdb.model.Recipe;
import com.zettenet.recipesdb.model.RecipeCategory;
import com.zettenet.recipesdb.model.RecipeIngredient;
import com.zettenet.recipesdb.model.RecipeIngredientEntry;
import com.zettenet.recipesdb.model.RecipeIngredientGroup;
import com.zettenet.recipesdb.model.RecipeMethod;
import com.zettenet.recipesdb.model.RecipeSource;
import com.zettenet.recipesdb.model.RecipeStep;
import com.zettenet.recipesdb.services.impl.InvalidClientException;
import com.zettenet.recipesdb.services.impl.InvalidDataException;
import com.zettenet.recipesdb.web.RecipesApplication;
import com.zettenet.recipesdb.web.components.GPanel;
import com.zettenet.recipesdb.web.components.formFields.CommaSeparatedSetTextField;
import com.zettenet.recipesdb.web.components.formFields.NewlineSeparatedListTextArea;
import com.zettenet.recipesdb.web.components.listEditor.ListEditor;
import com.zettenet.recipesdb.web.components.listEditor.ListEditorItem;
import com.zettenet.recipesdb.web.components.listEditor.ListEditorRemoveButton;
import com.zettenet.recipesdb.web.models.RecipeCategoriesModel;
import com.zettenet.recipesdb.web.models.RecipeCompoundPropertyModel;

public class RecipeEdit extends GPanel<Recipe> {
	private static final long serialVersionUID = -3696764516863823970L;
	
	final RecipeForm form;
	final TextField<Recipe> name;
	final TextField<Recipe> servings;
	final TextField<?> source;
	final CommaSeparatedSetTextField<RecipeCategory> categories;
	final ListEditor<RecipeIngredientEntry> ingredients;
	final Button addIngr;
	final TextArea<Object> methodPreNote;
	final TextArea<Object> methodPostNote;
	final NewlineSeparatedListTextArea<RecipeStep> methodSteps;
	final Button preview;
	final Button submit;
	
	final Long recipeId;
	
	private class RecipeForm extends Form<Recipe> {
		public RecipeForm(String id) {
			super(id);
		}

		private static final long serialVersionUID = 5675505033814059395L;

		@Override
		protected void onSubmit() {
			Recipe edited = parseRecipe();
			
			Long id = new Long(0);
			
			try {
				id = RecipesApplication.get().getRecipesDataService().saveOrUpdateRecipe("web", edited);
			} catch (InvalidClientException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InvalidDataException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InstantiationException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			PageParameters params = new PageParameters();
			params.add("view", id.toString());
			com.zettenet.recipesdb.web.pages.Recipe recipePage = new com.zettenet.recipesdb.web.pages.Recipe(params);
			setResponsePage(recipePage);
		}
		
		protected Recipe parseRecipe() {
			this.validate();
			ingredients.updateModel();
			
			Recipe edited;
			edited = new Recipe();
			edited.setName(name.getValue());
			edited.setServings(servings.getValue());
			edited.setSource(new RecipeSource(source.getValue()));
			edited.setCategories((Set<RecipeCategory>) categories.getConvertedInput());
			
			List<RecipeIngredientEntry> editedIngrs = (List<RecipeIngredientEntry>) ingredients.getModelObject();
			List<RecipeIngredientEntry> persistentIngrs = new ArrayList<RecipeIngredientEntry>();
			
			for (RecipeIngredientEntry editedIngr : editedIngrs) {
				if (editedIngr.getIngredient() != null && !editedIngr.getIngredient().getName().equals("")) {
					RecipeIngredientEntry newEditIngr = new RecipeIngredientEntry();
					
					if (editedIngr.getGroup() == null || editedIngr.getGroup().getName().equals("")) {
						editedIngr.setGroup(editedIngrs.get(editedIngrs.indexOf(editedIngr) -1).getGroup());
					}
					
					newEditIngr.setGroup(new RecipeIngredientGroup(editedIngr.getGroup().getName()));
					newEditIngr.setAmount(editedIngr.getAmount());
					newEditIngr.setIngredient(new RecipeIngredient(editedIngr.getIngredient().getName()));
					
					persistentIngrs.add(newEditIngr);
				}
			}
			
			edited.setIngredients(persistentIngrs);
			
			RecipeMethod editedMethod = new RecipeMethod();
			editedMethod.setNumberedSteps(true);
			editedMethod.setPreNote(methodPreNote.getValue() == "" ? methodPreNote.getValue() : null);
			editedMethod.setPostNote(methodPostNote.getValue() == "" ? methodPostNote.getValue() : null);
			editedMethod.setSteps((List<RecipeStep>) methodSteps.getConvertedInput());
			
			if (recipeId != null && !recipeId.equals(0)) {
					edited.setId(recipeId);
					editedMethod.setId(recipeId);
			}
			
			edited.setMethod(editedMethod);
			
			return edited;
		}
	}

	public RecipeEdit(String id, Long recipeId) {
		super(id, new RecipeCompoundPropertyModel(recipeId));
		
		this.recipeId = recipeId;

		this.form = new RecipeForm("recipe");

		this.name = new TextField<Recipe>("name");

		this.servings = new TextField<Recipe>("servings");

		this.source = new TextField<Object>("source",
				((CompoundPropertyModel<Recipe>) getModel())
						.bind("source.name"));

		this.categories = new CommaSeparatedSetTextField<RecipeCategory>(
				"categories", new RecipeCategoriesModel(getModel()), RecipeCategory.class);

		this.ingredients = new ListEditor<RecipeIngredientEntry>("ingredients") {
			private static final long serialVersionUID = 8479383228620521189L;

			@Override
			protected void onPopulateItem(
					ListEditorItem<RecipeIngredientEntry> item) {
				TextField<RecipeIngredientGroup> ingrGroup = new TextField<RecipeIngredientGroup>(
						"ingrGroup", new PropertyModel<RecipeIngredientGroup>(
								item.getDefaultModelObject(), "group.name"));

				TextField<String> ingrAmt = new TextField<String>("ingrAmount",
						new PropertyModel<String>(item.getDefaultModelObject(),
								"amount"));

				TextField<RecipeIngredient> ingrName = new TextField<RecipeIngredient>(
						"ingrName",
						new PropertyModel<RecipeIngredient>(item
								.getDefaultModelObject(), "ingredient.name"));

				ListEditorRemoveButton ingrRem = new ListEditorRemoveButton(
						"ingrRem");
				ingrRem.setDefaultFormProcessing(false);

				item.add(ingrGroup);
				item.add(ingrAmt);
				item.add(ingrName);
				item.add(ingrRem);
			}
		};

		this.addIngr = new Button("ingrAdd") {
			private static final long serialVersionUID = -4029278243797321256L;

			@Override
			public void onSubmit() {
				RecipeEdit.this.ingredients
						.addItem(new RecipeIngredientEntry());
			}
		};
		this.addIngr.setDefaultFormProcessing(false);

		this.methodPreNote = new TextArea<Object>("methodPreNote",
				((CompoundPropertyModel<Recipe>) getModel()).bind("method.preNote"));
		this.methodPostNote = new TextArea<Object>("methodPostNote",
				((CompoundPropertyModel<Recipe>) getModel()).bind("method.postNote"));

		this.methodSteps = new NewlineSeparatedListTextArea<RecipeStep>(
				"methodSteps",
				new PropertyModel<Collection<RecipeStep>>(getModel(), "method.steps"),
				RecipeStep.class);
		
		this.preview = new Button("preview") {
			private static final long	serialVersionUID	= -6274753494852145456L;
			
			public void onSubmit() {
				Recipe edited = form.parseRecipe();
				
				CompoundPropertyModel<Recipe> editedRecipeModel = new CompoundPropertyModel<Recipe>(edited);
				com.zettenet.recipesdb.web.pages.Recipe recipePage = new com.zettenet.recipesdb.web.pages.Recipe(editedRecipeModel);
				setResponsePage(recipePage);
			}
		};
		this.preview.setDefaultFormProcessing(false);

		this.submit = new Button("submit");

		this.form.add(this.name);
		this.form.add(this.servings);
		this.form.add(this.source);
		this.form.add(this.categories);
		this.form.add(this.ingredients);
		this.form.add(this.addIngr);
		this.form.add(this.methodPreNote);
		this.form.add(this.methodPostNote);
		this.form.add(this.methodSteps);
		this.form.add(this.preview);
		this.form.add(this.submit);
		add(this.form);
	}

}
