package com.zettenet.recipesdb.web.components.pageContents;

import java.util.List;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;

import com.zettenet.recipesdb.model.Recipe;
import com.zettenet.recipesdb.model.RecipeCategoryEntry;
import com.zettenet.recipesdb.model.RecipeIngredientEntry;
import com.zettenet.recipesdb.model.RecipeStep;
import com.zettenet.recipesdb.web.components.GPanel;
import com.zettenet.recipesdb.web.components.ListViewOfSet;
import com.zettenet.recipesdb.web.models.RecipeCompoundPropertyModel;

public class RecipeView extends GPanel<Recipe> {
	private static final long serialVersionUID = 8477678066029603561L;

	public RecipeView(String id, Long recipeId) {
		super(id, new RecipeCompoundPropertyModel(recipeId));
		viewRecipe();
	}

	public RecipeView(String id, IModel<Recipe> model) {
		// Should only be used for previewing edited/new recipes
		super(id, model);
		viewRecipe();
	}

	private void viewRecipe() {
		Label name = new Label("name");
		add(name);

		Label servings = new Label("servings");
		add(servings);

		Label source = new Label("source",
				((CompoundPropertyModel<Recipe>) getModel())
						.bind("source.name"));
		add(source);

		ListViewOfSet<RecipeCategoryEntry> categories = new ListViewOfSet<RecipeCategoryEntry>("categories") {
			private static final long serialVersionUID = 5314444558484180398L;

			@Override
			protected void onPopulateItem(RecipeCategoryEntry item) {
				add(new Label(newChildId(),
						new PropertyModel<RecipeCategoryEntry>(item, "category.name")));
			}
		};
		categories.setRenderBodyOnly(true);
		add(categories);

		ListView<RecipeIngredientEntry> ingredients = new ListView<RecipeIngredientEntry>(
				"ingredients") {
			private static final long serialVersionUID = 6569730182047385305L;
			
			private String oldIngrGroupName = "Main";

			@Override
			protected void populateItem(ListItem<RecipeIngredientEntry> item) {
				Label ingrGroup = new Label("ingrGroup",
						new PropertyModel<RecipeIngredientEntry>(item.getDefaultModelObject(), "group.name"));
				ingrGroup.setRenderBodyOnly(true);
				
				Label ingrAmt = new Label("ingrAmount",
						new PropertyModel<RecipeIngredientEntry>(item.getDefaultModelObject(), "amount"));
				ingrAmt.setRenderBodyOnly(true);
				
				Label ingrName = new Label("ingrName",
						new PropertyModel<RecipeIngredientEntry>(item.getDefaultModelObject(), "ingredient.name"));
				ingrName.setRenderBodyOnly(true);

				ingrGroup.setVisible(!item.getModelObject().getGroup()
						.getName().equals(this.oldIngrGroupName));
				this.oldIngrGroupName = item.getModelObject().getGroup()
						.getName();

				item.add(ingrGroup);
				item.add(ingrAmt);
				item.add(ingrName);
				item.setRenderBodyOnly(true);
			}
		};
		ingredients.setRenderBodyOnly(true);
		add(ingredients);

		Label methodPreNote = new Label("methodPreNote",
				((CompoundPropertyModel<Recipe>) getModel()).bind("method.preNote"));
		methodPreNote
				.setVisible((getModelObject().getMethod().getPreNote() != null)
						&& !getModelObject().getMethod().getPreNote().equals(""));
		add(methodPreNote);

		Label methodPostNote = new Label("methodPostNote",
				((CompoundPropertyModel<Recipe>) getModel()).bind("method.postNote"));
		methodPostNote
				.setVisible((getModelObject().getMethod().getPostNote() != null)
						&& !getModelObject().getMethod().getPostNote().equals(""));
		add(methodPostNote);

		final boolean numberedSteps = getModelObject().getMethod()
				.isNumberedSteps();

		ListView<RecipeStep> methodSteps = new ListView<RecipeStep>(
				"methodSteps", new PropertyModel<List<RecipeStep>>(getModelObject(), "method.steps")) {
			private static final long serialVersionUID = 5371722983405888036L;
			
			private int stepNum = 1;

			@Override
			protected void populateItem(ListItem<RecipeStep> item) {
				Label stepLabel = new Label("methodStep", "");
				if (!item.getModelObject().getStep().equals("")) {
					stepLabel = numberedSteps ?
							new Label("methodStep", this.stepNum + ". " + item.getModelObject().getStep()) :
							new Label("methodStep", item.getModelObject().getStep());
					this.stepNum++;
				}
				item.add(stepLabel);
			}
		};
		
		add(methodSteps);
	}
}
