package com.zettenet.recipesdb.test.harness;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import com.zettenet.recipesdb.model.Recipe;
import com.zettenet.recipesdb.model.RecipeCategory;
import com.zettenet.recipesdb.model.RecipeIngredient;
import com.zettenet.recipesdb.model.RecipeIngredientEntry;
import com.zettenet.recipesdb.model.RecipeIngredientGroup;
import com.zettenet.recipesdb.model.RecipeMethod;
import com.zettenet.recipesdb.model.RecipeSource;
import com.zettenet.recipesdb.model.RecipeStep;


public final class RecipeTestData {
	public static final String ingrWrong = "satl";
	public static final String ingrRight = "salt";
	public static final String catWrong = "Desert";
	public static final String catRight = "Dessert";
	public static final String srcWrong = "Magamazine";
	public static final String srcRight = "Magazine";
	
	public static final String rcpName = "Pancakes";
	
	public static final RecipeIngredient ingrFlour = new RecipeIngredient("flour");
	public static final RecipeIngredient ingrEggs = new RecipeIngredient("eggs");
	public static final RecipeIngredient ingrMilk = new RecipeIngredient("milk");
	public static final RecipeIngredient ingrSugar = new RecipeIngredient("sugar");
	public static final RecipeIngredient ingrLemonJuice = new RecipeIngredient("lemon juice");
	
	public static final RecipeCategory catMain = new RecipeCategory("Main course");
	public static final RecipeCategory catMainCat = new RecipeCategory("Main course for cats");
	public static final RecipeCategory catDessert = new RecipeCategory(catWrong);
	
	public static final RecipeIngredientGroup ingrGroupMain = new RecipeIngredientGroup("Main");
	public static final RecipeIngredientGroup ingrGroupServe = new RecipeIngredientGroup("To serve");
	
	public static final RecipeSource srcMagazine = new RecipeSource(srcWrong);
	public static final RecipeSource srcNewspaper = new RecipeSource("Newspaper");
	
	public static final RecipeIngredient ingrBadSalt = new RecipeIngredient(ingrWrong);
	
	public static Recipe getFullRecipe() {
		Recipe rcpPancakes = new Recipe();
		rcpPancakes.setName(rcpName);
		rcpPancakes.setServings("6");
		rcpPancakes.setSource(srcMagazine);
		
		Set<RecipeCategory> categories = new HashSet<RecipeCategory>();
		categories.add(catMain);
		categories.add(catDessert);
		rcpPancakes.setCategories(categories);
		
		RecipeMethod method = new RecipeMethod();
		method.setNumberedSteps(true);
		
		List<RecipeIngredientEntry> ingredients = new ArrayList<RecipeIngredientEntry>();
		ingredients.add(new RecipeIngredientEntry(0, "2 cups", ingrFlour, ingrGroupMain));
		ingredients.add(new RecipeIngredientEntry(1, "4", ingrEggs, ingrGroupMain));
		ingredients.add(new RecipeIngredientEntry(2, "2 cups", ingrMilk, ingrGroupMain));
		ingredients.add(new RecipeIngredientEntry(3, "1 tbsp", ingrSugar, ingrGroupMain));
		ingredients.add(new RecipeIngredientEntry(4, "pinch", ingrBadSalt, ingrGroupMain));
		ingredients.add(new RecipeIngredientEntry(5, "", ingrLemonJuice, ingrGroupServe));
		rcpPancakes.setIngredients(ingredients);
		
		List<RecipeStep> steps = new ArrayList<RecipeStep>();
		steps.add(new RecipeStep(0, "Mix the flour, eggs, milk, sugar and salt and beat until smooth."));
		steps.add(new RecipeStep(1, "Melt a small amount of butter in a frying pan or pancake pan, and keep it at a temperature just below the smoke point."));
		steps.add(new RecipeStep(2, "Fry pancakes of the desired size in the butter, turning once. If the pan dries out, add a little more butter."));
		steps.add(new RecipeStep(3, "Serve the pancakes with sugar and lemon juice to taste."));
		method.setSteps(steps);
		
		rcpPancakes.setMethod(method);
		
		return rcpPancakes;
	}
	
}
